@@ -28,6 +28,7 @@ with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <glib-object.h>
 #include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
+#include <gio/gio.h>
 #include <libdbusmenu-gtk/menu.h>
 
 #include <dbus/dbus-glib.h>
@@ -216,12 +217,59 @@ get_icon (IndicatorObject * io)
 	return INDICATOR_SESSION(io)->status_image;
 }
 
+static void
+user_property_change (DbusmenuMenuitem * item, const gchar * property, const GValue * value, gpointer user_data)
+{
+	if (g_strcmp0(property, USER_ITEM_PROP_LOGGED_IN) == 0) {
+		if (g_value_get_boolean(value)) {
+			gtk_widget_show(GTK_WIDGET(user_data));
+		} else {
+			gtk_widget_hide(GTK_WIDGET(user_data));
+		}
+	}
+	return;
+}
+
 /* Builds an item with a hip little logged in icon. */
 static gboolean
 new_user_item (DbusmenuMenuitem * newitem, DbusmenuMenuitem * parent, DbusmenuClient * client)
 {
 	GtkMenuItem * gmi = GTK_MENU_ITEM(gtk_menu_item_new());
-	GtkWidget * hbox = gtk_hbox_new(FALSE, 0);
+	gint padding = 0;
+	gtk_widget_style_get(GTK_WIDGET(gmi), "horizontal-padding", &padding, NULL);
+	GtkWidget * hbox = gtk_hbox_new(FALSE, padding);
+
+	GtkWidget * usericon = NULL;
+	const gchar * icon_name = dbusmenu_menuitem_property_get(newitem, USER_ITEM_PROP_ICON);
+	g_debug("Using user icon for '%s' from file: %s", dbusmenu_menuitem_property_get(newitem, USER_ITEM_PROP_NAME), icon_name);
+	if (icon_name != NULL && icon_name[0] != '\0') {
+		if (g_strcmp0(icon_name, USER_ITEM_ICON_DEFAULT) != 0 && g_file_test(icon_name, G_FILE_TEST_EXISTS)) {
+			gint width, height;
+			gtk_icon_size_lookup(GTK_ICON_SIZE_MENU, &width, &height);
+
+			GError * error = NULL;
+			GdkPixbuf * pixbuf = gdk_pixbuf_new_from_file_at_size(icon_name, width, height, &error);
+
+			if (error == NULL) {
+				usericon = gtk_image_new_from_pixbuf(pixbuf);
+				g_object_unref(pixbuf);
+			} else {
+				g_warning("Unable to load user icon '%s': %s", icon_name, error->message);
+				g_error_free(error);
+			}
+		}
+
+		if (usericon == NULL) {
+			GIcon * gicon = g_themed_icon_new_with_default_fallbacks("stock_person-panel");
+			usericon = gtk_image_new_from_gicon(gicon, GTK_ICON_SIZE_MENU);
+			g_object_unref(gicon);
+		}
+	}
+	if (usericon != NULL) {
+		gtk_misc_set_alignment(GTK_MISC(usericon), 0.0, 0.5);
+		gtk_box_pack_start(GTK_BOX(hbox), usericon, FALSE, FALSE, 0);
+		gtk_widget_show(usericon);
+	}
 
 	GtkWidget * label = gtk_label_new(dbusmenu_menuitem_property_get(newitem, USER_ITEM_PROP_NAME));
 	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
@@ -242,6 +290,8 @@ new_user_item (DbusmenuMenuitem * newitem, DbusmenuMenuitem * parent, DbusmenuCl
 
 	dbusmenu_gtkclient_newitem_base(DBUSMENU_GTKCLIENT(client), newitem, gmi, parent);
 
+	g_signal_connect(G_OBJECT(newitem), DBUSMENU_MENUITEM_SIGNAL_PROPERTY_CHANGED, G_CALLBACK(user_property_change), icon);
+
 	return TRUE;
 }
 
@@ -264,11 +314,20 @@ switch_property_change (DbusmenuMenuitem * item, const gchar * property, const G
 	GtkMenuItem * gmi = dbusmenu_gtkclient_menuitem_get(DBUSMENU_GTKCLIENT(user_data), item);
 	gchar * finalstring = NULL;
 	gboolean set_ellipsize = FALSE;
+	gboolean no_name_in_lang = FALSE;
+
+	/* TRANSLATORS: Translate the '1' below into anything other than
+	   '1' if "Switch From %s..." doesn't make sense in your language.
+	   Instead, the string "Switch User..." will be used. */
+	const gchar * translate = C_("session_menu:switchfrom", "1");
+	if (g_strcmp0(translate, "1") != 0) {
+		no_name_in_lang = TRUE;
+	}
 
 	/* If there's a NULL string of some type, then we want to
 	   go back to our old 'Switch User' which isn't great but
 	   eh, this error condition should never happen. */
-	if (value == NULL || g_value_get_string(value) == NULL || g_value_get_string(value)[0] == '\0') {
+	if (value == NULL || g_value_get_string(value) == NULL || g_value_get_string(value)[0] == '\0' || no_name_in_lang) {
 		finalstring = _("Switch User...");
 		set_ellipsize = FALSE;
 	}
