@@ -24,14 +24,13 @@ with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include <libdbusmenu-glib/menuitem.h>
 
-typedef void (*gdm_autologin_cb_t) (void);
-
 void screensaver_throttle (gchar * reason);
 void screensaver_unthrottle (void);
 
 gboolean will_lock_screen (void);
-void lock_screen (DbusmenuMenuitem * mi, gpointer data);
+void lock_screen (DbusmenuMenuitem * mi, guint timestamp, gpointer data);
 gboolean lock_screen_setup (gpointer data);
-void lock_screen_gdm_cb_set (gdm_autologin_cb_t cb);
+
+void lock_screen_update_item (DbusmenuMenuitem * mi);
 
 #endif /* LOCK_HELPER_H__ */
